import { Id } from "../Id";
import { Tree } from "./Tree";
export declare enum NodeType {
    File = 0,
    Directory = 1
}
export type TreeNode = MutableNode | ImmutableDirectory;
export type BaseOpts = {
    id: Id;
    type: NodeType;
    name?: string;
    parentId?: Id;
};
export type FileOpts = BaseOpts & {
    type: NodeType.File;
    parentId: Id;
    name: string;
};
export type DirectoryOpts = BaseOpts & {
    type: NodeType.Directory;
    parentId: Id;
    name: string;
};
export type ImmutableDirectoryOpts = BaseOpts & {
    type?: NodeType.Directory;
};
export type MutableOpts = BaseOpts & {
    name: string;
    parentId: Id;
};
export type Opts = MutableOpts | ImmutableDirectoryOpts;
/** Create an FSNode from a TreeNode in {@link TreeReplica}. */
export declare function createNode(opts: Opts, tree: Tree): TreeNode;
export declare class BaseNode {
    /** Unique id of the node. */
    readonly id: Id;
    /** Type of the node. */
    readonly type: NodeType;
    /** Parent id of the node. */
    readonly parentId?: Id;
    /** Name of the FS entry. */
    readonly name?: string;
    /** A reference to the tree. */
    protected tree: Tree;
    constructor(opts: BaseOpts, tree: Tree);
    get parent(): DirNode | undefined;
    get path(): string;
    /** Returns whether this node is a {@link File}. */
    isFile(): this is InstanceType<typeof File>;
    /**
     * Returns whether this node is a {@link DirNode}.
     */
    isDirNode(): this is DirNode;
    /**
     * Returns whether this node is a {@link Directory}.
     *
     * This does *not* include immutable directories like {@link ROOT}.
     * To identify {@link Directory} or {@link ImmutableDirectory}, use {@link isDirNode}.
     */
    isDirectory(): this is InstanceType<typeof Directory>;
    /** Returns whether this node is a {@link MutableNode}. */
    isMutable(): this is InstanceType<typeof MutableNode>;
    /** Returns whether this node is an {@link ImmutableDirectory}. */
    isImmutableDirectory(): this is InstanceType<typeof ImmutableDirectory>;
}
/**
 * A mixin providing methods for node modification such as moving, deleting,
 * and renaming.
 */
export declare class MutableNode extends BaseNode {
    parentId: Id;
    name: string;
    /** This node's parent node. */
    get parent(): DirNode;
    /** Move this node to the provided directory node. */
    move(parent: DirNode, name?: string): void;
    /**
     * Rename this node to the provided name.
     * Optionally, move it to the provided parent directory node.
     */
    rename(name: string): void;
    /** Delete this node. */
    delete(): void;
}
/** A mixin providing methods specific to directories. */
export declare class DirNode extends BaseNode {
    type: NodeType.Directory;
    /** Returns an array of a node's children. */
    get children(): MutableNode[];
    /** Find a child node by name */
    getChildByName(name: string): MutableNode | undefined;
    /** Create a file within this directory. */
    createFileWithId(id: Id, name: string): File;
    createFile(name: string): File;
    /** Create a directory within this directory. */
    createDirectoryWithId(id: Id, name: string): Directory;
    /** Create a directory within this directory. */
    createDirectory(name: string): Directory;
}
export declare class File extends MutableNode {
    type: NodeType.File;
}
declare const Directory_base: import("ts-mixer/dist/types/types").Class<any[], MutableNode & DirNode, typeof MutableNode & typeof DirNode>;
export declare class Directory extends Directory_base {
}
export declare class ImmutableDirectory extends DirNode {
    parentId: undefined;
    name: undefined;
    get parent(): undefined;
}
export {};
