import { TextOperation } from "./TextOperation";
import { OperationReason } from "./types";
export interface IncomingOperationEvent {
    revision: number;
    operation: TextOperation;
    reason: OperationReason;
}
export interface OutgoingOperationEvent {
    revision: number;
    operation: TextOperation;
}
export interface OperationAckEvent {
    revision: number;
}
export interface ContentChangeEvent {
    newContent: string;
}
export type FnSendOperation = (revision: number, operation: TextOperation) => void;
export type FnSendOperationAck = (revision: number) => void;
export declare class Client {
    private incomingOperationEmitter;
    onIncomingOperation: import("../event").Event<IncomingOperationEvent>;
    private operationAckEmitter;
    onOperationAck: import("../event").Event<OperationAckEvent>;
    private outgoingOperationEmitter;
    onOutgoingOperation: import("../event").Event<OutgoingOperationEvent>;
    private documentChangeEmitter;
    onDocumentChange: import("../event").Event<ContentChangeEvent>;
    private _document;
    private _serverDocument;
    revision: number;
    private sendOperation;
    private sendOperationAck;
    private pendingOperation;
    private queuedOperation;
    constructor(document: string, revision: number, sendOperation: FnSendOperation, sendOperationAck: FnSendOperationAck);
    /** Check if client has pending operation, used for testing */
    get hasPendingOperation(): boolean;
    /** Check if client has pending operation, used for testing */
    get hasQueuedOperation(): boolean;
    /** returns the current state of the document */
    get document(): string;
    /** sets the content of the document and emits a documentChange event */
    private set document(value);
    /** Enqueue operation to send to the server */
    private enqueueOperation;
    private transformServerOperation;
    private sendPendingOperation;
    private sendQueuedOperation;
    /**
     * Resync with the server using the new content and last known revision
     *
     * Transforms the queued and pending operation on top of the diff between
     * our last known state and the expected state
     **/
    syncServerDocument(serverDocument: string, newRevision: number): void;
    /**
     * If the OTClient errors on applying new operations from
     * the client, we can recover the state of the client document
     * by calling this method. It can potentially override server changes, but it is
     * more important that users do not loose any work
     */
    applyClientDocument(clientDocument: string): void;
    /**
     * Apply an operation originating from this client
     *
     * @param clientOperation the operation to apply to the document
     */
    applyClientOperation(clientOperation: TextOperation): void;
    /**
     * Apply operation coming from the ot server
     *
     * @param serverOperation the operation from the server that should be applied to the document
     * @param reason the reason for the operation to occur
     */
    applyServerOperation(op: TextOperation, reason: OperationReason): void;
    /**
     * On successful passing of an operation to the server. Any queued
     * operations will now be sent. The "onOperationAck" event is emitted when
     * there are no more pending operations or after we send any queued operations
     */
    serverAck(): void;
    dispose(): void;
}
